from collections.abc import Callable
from typing import Any, TypeVar

_F = TypeVar("_F", bound=Callable[..., Any])

def cache_page(timeout: float | None, *, cache: Any | None = ..., key_prefix: Any | None = ...) -> Callable: ...
def cache_control(**kwargs: Any) -> Callable: ...
def never_cache(view_func: _F) -> _F: ...
