from typing import Any

class WKTAdapter:
    wkt: Any
    srid: Any
    def __init__(self, geom: Any) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
