__all__ = ["UnknownTimeZoneError", "InvalidTimeError", "AmbiguousTimeError", "NonExistentTimeError"]

class Error(Exception): ...
class UnknownTimeZoneError(KeyError, Error): ...
class InvalidTimeError(Error): ...
class AmbiguousTimeError(InvalidTimeError): ...
class NonExistentTimeError(InvalidTimeError): ...
