from nwon_django_toolbox.models.update_validation.allow_updates_for_model import (
    allow_updates_for_model,
)
from nwon_django_toolbox.models.update_validation.validate_model_update import (
    validate_model_update,
)
from nwon_django_toolbox.typings import AllowedUpdate

__all__ = ["allow_updates_for_model", "validate_model_update", "AllowedUpdate"]
